﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class Manager : MonoBehaviour
{
    [Header("Creation Variables")]
    #region VariablesForCreation
    public bool isCreating;
    public float minWaitToCreate;
    public float maxWaitToCreate;
    public float randomNumberWaiResult;
    public int pooledAmount;
    public GameObject pooledObject;
    List<GameObject> pooledObjects = new List<GameObject>();
    public Transform objectPoolHolder;
    public bool willGrow;
    #endregion
    [Header("Movement Variables")]
    #region VariablesForMovement
    public List<Transform> wayPoints = new List<Transform>();
    public float minWaitStop;
    public float maxWaitStop;
    public float minSpeed;
    public float maxSpeed;
    public float speedResult;
    #endregion
    [Header("Calculation Variables")]
    #region CalculationVariables
    public int finishedGameobjectCount;
    public float timeElapsedFromStart;
    public float showAverage;
    public Text infoText;
    #endregion

    void Start()
    {
        for (int i = 0; i < pooledAmount; i++)
        {
            GameObject obj = (GameObject)Instantiate(pooledObject);
            obj.transform.parent = objectPoolHolder;
            obj.SetActive(false);
            pooledObjects.Add(obj);
        }
    }

    void Update()
    {
        if (!isCreating && GetPooledObject() != null)
        {
            StartCoroutine(Creation());
        }
        timeElapsedFromStart = Time.time;
        CalculateAverage(timeElapsedFromStart, finishedGameobjectCount);
        showAverage = CalculateAverage(timeElapsedFromStart, finishedGameobjectCount);
        infoText.text = "Time Elapse = " + " " + timeElapsedFromStart.ToString("F2") + " " +
            "Average Production Time Per Item" + " " + showAverage.ToString("F2") + " "
            + "Finished Gameobject Count" + " " + finishedGameobjectCount.ToString("F2");
    }

    public GameObject GetPooledObject()
    {
        for (int i = 0; i < pooledObjects.Count; i++)
        {
            if (!pooledObjects[i].activeInHierarchy)
            {
                return pooledObjects[i];
            }
        }
        if (willGrow)
        {
            GameObject holder = GameObject.Find("PoolHolder");
            GameObject obj = (GameObject)Instantiate(pooledObject);
            obj.SetActive(false);
            obj.transform.parent = holder.transform;
            pooledObjects.Add(obj);
            return obj;
        }
        return null;
    }

    public void GenerateRandomWaitNumberToCreate(float a, float b)
    {
        randomNumberWaiResult = Random.Range(a, b);
    }

    IEnumerator Creation()
    {
        isCreating = true;
        GenerateRandomWaitNumberToCreate(minWaitToCreate, maxWaitToCreate);
        yield return new WaitForSeconds(randomNumberWaiResult);
        GameObject obj = GetPooledObject();
        obj.SetActive(true);
        obj.transform.localPosition = Vector3.zero;
        obj.transform.localEulerAngles = Vector3.zero;
        StartCoroutine(Movement(obj));
        isCreating = false;
    }

    /*Now starts Movement Management-----------------------------------------------------*/
    public float GenerateRandomWaitNumberToMove(float a, float b)
    {
        return Random.Range(a, b);
    }

    IEnumerator Movement(GameObject go)
    {
        int wpIndex = 0;
        while (go.activeSelf)
        {
            Vector3 dir = wayPoints[wpIndex].transform.position - go.transform.position;
            go.transform.LookAt(wayPoints[wpIndex].transform);
            speedResult = Random.Range(minSpeed, maxSpeed);
            while (Vector3.Distance(go.transform.position, wayPoints[wpIndex].transform.position) >= 0.2f)
            {
                go.transform.Translate(dir.normalized * Time.deltaTime * speedResult, Space.World);
                yield return null;
            }
            yield return new WaitForSeconds(GenerateRandomWaitNumberToMove(minWaitStop, maxWaitStop));
            if (wpIndex >= wayPoints.Count - 1)
            {
                finishedGameobjectCount++;
                wpIndex = 0;
                go.SetActive(false);
            }
            else
            {
                wpIndex++;
            }
        }
    }
    //Now Calculating Average Production Time-----------------------------------------------
    public float CalculateAverage(float a, int b)
    {
        return a / b;
    }
}


